@@ -134,7 +134,7 @@ class productsCsvImporter
   {
     $this->_line = $tmp;
     $this->_linenum++;
-    $this->_record = explode($this->_delim,$this->_line);
+    $this->_record = cge_array::smart_explode($this->_line,$this->_delim);
     return TRUE;
   }
 
@@ -276,7 +276,10 @@ class productsCsvImporter
   function _getField($name,$dflt = FALSE)
   {
     $idx = $this->_hasField($name);
-    if( $idx === FALSE ) return $dflt;
+    if( $idx === FALSE )
+      {
+	return $dflt;
+      }
     $val = trim($this->_record[$idx],'\'"');
     return $val;
   }
@@ -387,6 +390,7 @@ class productsCsvImporter
 	$fields = $this->_getFieldNamesLike('#FIELD');
 	if( is_array($fields) )
 	  {
+	    $field_types = array_keys(product_utils::get_field_types());
 	    $fquery = 'SELECT id FROM '.cms_db_prefix().'module_products_fielddefs WHERE name = ?';
 	    $oquery = 'SELECT max(item_order) + 1 FROM '.cms_db_prefix().'module_products_fielddefs';
 	    $iquery = 'INSERT INTO '.cms_db_prefix()."module_products_fielddefs 
@@ -399,6 +403,11 @@ class productsCsvImporter
 		$type = trim(trim($tmp[1]),'"\'');
 		$name = trim(trim($tmp[2]),'"\'');
 		if( empty($type) || empty($name) ) continue;
+		if( !in_array($type,$field_types) )
+		  {
+		    $this->_setError('Invalid field type: '.$type.' for field '.$name);
+		    return FALSE;
+		  }
 		$t_id = $db->GetOne($fquery,array($name));
 		if( !$t_id )
 		  {
@@ -720,6 +729,7 @@ class productsCsvImporter
     $fields = $this->_getFieldNamesLike('#FIELD');
     if( is_array($fields) )
       {
+	$field_types = array_keys(product_utils::get_field_types());
 	$fquery = 'SELECT id FROM '.cms_db_prefix().'module_products_fielddefs WHERE name = ?';
 	foreach( $fields as $one )
 	  {
@@ -727,7 +737,16 @@ class productsCsvImporter
 	    if( count($tmp) != 3 ) continue;
 	    $type = trim(trim($tmp[1]),'"\'');
 	    $name = trim(trim($tmp[2]),'"\'');
-	    if( empty($type) || empty($name) ) continue;
+	    if( empty($type) || empty($name) )
+	      {
+		$this->_setError('Invalid field definition: '.$one);
+		return FALSE;
+	      }
+	    if( !in_array($type,$field_types) )
+	      {
+		$this->_setError('Invalid field type: '.$type);
+		return FALSE;
+	      }
 	    $t_id = $db->GetOne($fquery,array($name));
 	    if( !$t_id )
 	      {
@@ -750,7 +769,9 @@ class productsCsvImporter
     // according to the header.
     if( $this->_getFieldCount() != $this->_getDefCount() )
       {
-	$this->_setError('invalid field count');
+	debug_display($this->_defs_byindex);
+	debug_display($this->_record); die();
+	$this->_setError('invalid field count ('.$this->_getFieldCount().' of expected '.$this->_getDefCount().')');
 	return FALSE;
       }
 
@@ -824,6 +845,65 @@ class productsCsvImporter
     return $res;
   }
 
+  protected function is_new_record($line)
+  {
+    // a function to test if this line denotes the start of a new record.
+    $valid = array('#FLAG','P','"#FLAG"','"P"');
+
+    foreach( $valid as $one )
+      {
+	$one .= $this->_delim;
+	if( startswith($line,$one) ) return TRUE;
+      }
+    return FALSE;
+  }
+
+
+  private function my_fgets($fh)
+  {
+    if( !$fh ) return;
+    $pos1 = ftell($fh);
+    
+    $line = fgets($fh);
+    if( strpos($line,"\r") === FALSE )
+      {
+	return $line;
+      }
+    
+    // line is probably a crappy mac line.
+    $len1 = strlen($line);
+    $pos = strpos($line,"\r");
+    
+    $line = substr($line,0,$pos);
+    fseek($fh,($len1 - $pos -1 ) * -1,SEEK_CUR);
+    return $line;    
+  }
+
+
+  public function get_unparsed_record($fh)
+  {
+    $line = '';
+    while( !feof($fh) )
+      {
+	$pos = ftell($fh);
+	$tdata = $this->my_fgets($fh);
+	if( !$this->is_new_record($tdata) )
+	  {
+	    $line .= $tdata;
+	    continue;
+	  }
+
+	if( strlen($line) > 0 )
+	  {
+	    fseek($fh,$pos);
+	    break;
+	  }
+	$line = $tdata;
+      }
+
+    return $line;
+  }
+
 } // end of class
 
 
